#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/pgmspace.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/wdt.h>
#include "typedefs.h"

/*  */
#define CTRL_PORT          PORTB
#define CTRL_PORT_PIN      PINB
#define CTRL_PORT_DIR      DDRB
#define CTRL_PIN_POWER_ON  PB0               //      pin 1
#define CTRL_PIN_PTT       PB1               //      pin 2
#define CTRL_PIN_LOCK      PB4               //      pin 5

#define CTRL_PORT2         PORTD
#define CTRL_PORT2_PIN     PIND
#define CTRL_PORT2_DIR     DDRD
#define CTRL_PIN_MODE      PD3               //      pin 17

#define LCD_PORT           PORTC
#define LCD_PORT_DIR       DDRC
#define LCD_PIN0           PC0
#define LCD_PIN1           PC1
#define LCD_PIN2           PC2
#define LCD_PIN3           PC3
#define LCD_PIN_STROBE     PC4
#define LCD_PIN_RS         PC5

#define SYNTH_PORT         PORTD
#define SYNTH_PORT_DIR     DDRD     //  ATmega16A|  /  LC72131 (LC72131M)
#define SYNTH_PIN_CE       PD0      //      pin 14     |           pin 3 (2)
#define SYNTH_PIN_DI       PD1      //      pin 15     |           pin 4 (3)
#define SYNTH_PIN_CL       PD2      //      pin 16     |           pin 5 (4)

#define ADC_PORT           PORTA
#define ADC_PORT_DIR       DDRA
#define ADC_VCC_CONV_CONST     0.191   //       
#define ADC_OUTPUT_POWER_CONST 0.27   // -        

/*   */
void IOinit(void);
void nop(void);
void StartTCNT1(void);
void StopTCNT1(void);
void Wait(UINT32 delay);
void Wait2(UINT32 delay);
void Wait_us(UINT16 delay);
void Wait_ms(UINT16 delay);

void Power_ON(void);
void Power_OFF(void);
void PTT_ON(void);
void PTT_OFF(void);

void InitSynthPins(void);
void ProgramSynth(void);
void send_word(UINT32 word);
void SetCLLow(void);
void SetCLHigh(void);
void SetDILow(void);
void SetDIHigh(void);
void SetCELow(void);
void SetCEHigh();

void LCDPortInit(void);
void SET_E(void);
void CLR_E(void);
void SET_A0(void);
void CLR_A0(void);
void LCD_write_cmd(UINT8);
void LCD_write_data(UINT8);
void LCD_write(UINT8 value, UINT8 cd);
void lcd_print_str(char *text);
void cls(void);
void LCDinit(void);
void Locate(UINT8 line, UINT8 pos);
bool GetTemp(char *temperature);
UINT16 GetVolt(char *volt);
void buff_print_dec(char **buffer, UINT8 ciffer);
void buff_print_volts(char **buffer, UINT8 volts);
void lcd_print_dec(char *text, UINT8 ciffer);
void lcd_print_dec_float(char *text, UINT16 ciffer, UINT8 pos);
void lcd_print_volts(UINT8 volts);
UINT16 GetADC (UINT8 channel);
void ADCwork(void);
void ADCInit(void);
void PrintInfo1(char letter);
void PrintInfo2(bool result_temp, char* p_t, bool result_volt, char * p_volt);
void PrintLockUnlock(void);
void LocateAndPrint(UINT8 line, UINT8 pos, char* text);
void TestMode(void);

/*        */
void   OWIinit(void);
void   WriteBit1(void);
void   WriteBit0(void);
UINT8  ReadBit(void);
bool   DetectPresence(void);
void   SendByte(UINT8);
UINT8  ReceiveByte(void);
void   ReadRom(UINT8 * p_rom);
void   MatchRom(UINT8 * p_rom);
UINT8  ComputeCRC8(UINT8 byte, UINT8 crc);
UINT8  CalcCRC(UINT8 *p_array, UINT8 array_length);
void   ReadScratchpad(UINT8 * scratchpad);
INT16  ReadTemp(void);

/*      One Wire Interface */

/*    
#define OWI_DELAY_A   6
#define OWI_DELAY_B   64
#define OWI_DELAY_C   60
#define OWI_DELAY_D   10
#define OWI_DELAY_E   9
#define OWI_DELAY_F   55
#define OWI_DELAY_G   0
#define OWI_DELAY_H   480
#define OWI_DELAY_I   70
#define OWI_DELAY_J   410
*/

//    2.5       2 
#define OWI_DELAY_A   2
#define OWI_DELAY_B   26
#define OWI_DELAY_C   24
#define OWI_DELAY_D   4
#define OWI_DELAY_E   4
#define OWI_DELAY_F   22
#define OWI_DELAY_G   0
#define OWI_DELAY_H   192
#define OWI_DELAY_I   28
#define OWI_DELAY_J   164

/*
#define OWI_DELAY_A   2
#define OWI_DELAY_B   16
#define OWI_DELAY_C   15
#define OWI_DELAY_D   2
#define OWI_DELAY_E   2
#define OWI_DELAY_F   14
#define OWI_DELAY_G   0
#define OWI_DELAY_H   160
#define OWI_DELAY_I   17
#define OWI_DELAY_J   102
*/

//    2  - 
/*#define OWI_DELAY_A   2
#define OWI_DELAY_B   21
#define OWI_DELAY_C   20
#define OWI_DELAY_D   3
#define OWI_DELAY_E   3
#define OWI_DELAY_F   18
#define OWI_DELAY_G   0
#define OWI_DELAY_H   120
#define OWI_DELAY_I   23
#define OWI_DELAY_J   136*/

/* -  
#define OWI_DELAY_A   3
#define OWI_DELAY_B   32
#define OWI_DELAY_C   30
#define OWI_DELAY_D   5
#define OWI_DELAY_E   5
#define OWI_DELAY_F   23
#define OWI_DELAY_G   0
#define OWI_DELAY_H   240
#define OWI_DELAY_I   35
#define OWI_DELAY_J   205
*/

#define OWI_CMD_READ_ROM             0x33  //Identification
#define OWI_CMD_SKIP_ROM             0xCC  //Skip addressing
#define OWI_CMD_MATCH_ROM            0x55  //Address specific device
#define OWI_CMD_SEARCH_ROM           0xF0  //Obtain IDs of all devices on the bus
#define OWI_CMD_OVERDRIVE_SKIP_ROM   0x3C  //Overdrive version of SKIP ROM
#define OWI_CMD_OVERDRIVE_MATCH_ROM  0x69  //Overdrive version of MATCH ROM

#define OWI_ROM_SEARCH_FINISHED      0x00
#define OWI_ROM_SEARCH_FAILED        0xff

#define DS18B20_START_CONVERSION     0x44
#define DS18B20_READ_SCRATCHPAD      0xBE
#define DS18B20_SKIP_ROM             0xCC
#define DS18B20_ROM_SEARCH           0xF0

#define DDRx_OWI_PORT  DDRC
#define PORTx_OWI_PORT PORTC
#define PINx_OWI_PORT  PINC

#define OWI_LINE            PC7               // 1-wire 
#define DEFAULT_TEMPERATURE 0x0550            //    DS18B20 (+85.0 C)
#define T99_9_TEMPERATURE   0x063F            //    (+99.9 C)

/*       */
#define lcd_print_char(arg) LCD_write_data(arg)

//     
#define     MORSE_DOT_LENGTH             1

//     
#define     MORSE_COEFF_DASH             3.0

// 
#define     SPACE                        0x20

//  
#define     EXCLAMATION_MARK             0x21

//    ,    
#define     MORSE_DASH_LENGTH            MORSE_COEFF_DASH*MORSE_DOT_LENGTH

//   
#define     MORSE_PAUSE_WITHIN_SIGN      MORSE_DOT_LENGTH

//  
#define     MORSE_PAUSE_BETWEEN_SIGNES   MORSE_DASH_LENGTH

//     (9 )
#define     MORSE_PAUSE_BETWEEN_WORDS    7*MORSE_DOT_LENGTH

#define nop()  __asm__ volatile("nop");
#define _WDR() __asm__ volatile("wdr");

typedef struct
{
  float freq_step;
  UINT8 freq_value;
} FREQ_CONST;
