/*    LCD  */

#include "common_defs.h"

const UINT8   dec_mass_2[] = {10, 1};
const UINT16  dec_mass_5[] = {10000, 1000, 100, 10, 1};

const UINT8 PROGMEM temperature_mass[] = {'0', '1', '1', '2', '3', '3', '4', '4', '5', '6', '6', '7', '8', '8', '9', '9'};

/*****************************
 *      *
 *****************************/
void SET_E(void)
{
   LCD_PORT |= _BV(LCD_PIN_STROBE);
}

/*****************************
 *          *
 *****************************/
void CLR_E(void)
{
   LCD_PORT &= ~_BV(LCD_PIN_STROBE);
}

/*************************************
 *  RS(A0) (/) *
 *************************************/
void SET_RS(void)
{
   LCD_PORT |= _BV(LCD_PIN_RS);
}

/*************************************
 *    RS(A0) (/)   *
 *************************************/
void CLR_RS(void)
{
   LCD_PORT &= ~_BV(LCD_PIN_RS);
}

/*****************************
 *      *
 * cd != 0 -           *
 * cd == 0 -          *
 *****************************/
void LCD_write(UINT8 value, UINT8 cd)
{
   if (cd)
      SET_RS();
   else
      CLR_RS();

   LCD_PORT ^= ((value >> 4) & 0x0F);
   SET_E();
   CLR_E();
   LCD_PORT &= 0xF0;

   LCD_PORT ^= (value & 0x0F);
   SET_E();
   CLR_E();
   LCD_PORT &= 0xF0;
   Wait_us(40);
}

/*****************************
 *     *
 *****************************/
void LCD_write_data(UINT8 data)
{
   LCD_write(data, 1);
}

/*****************************
 *     *
 *****************************/
void LCD_write_cmd(UINT8 cmd)
{
   LCD_write(cmd, 0);
}

/***********************************
 *         *
 ***********************************/
void lcd_print_str(char *text)
{
   while (*text)
   {
      LCD_write_data((UINT8)*text++);
   }
}

/**********************
 *   *
 **********************/
void cls(void)
{
   LCD_write_cmd(0x01);     //         
   Wait_ms(2);              //
}

/*********************
 *  LCD *
 *********************/
void LCDinit(void)
{
   CLR_E(); Wait_ms(40);
   CLR_RS(); LCD_PORT |= 0x03; //   (8 ) (0x03 -    0x30) -       8-   
   Wait_us(1);  //    (tAS)
   SET_E();                //     (tDSW)
   CLR_E(); Wait_us(40);   //  
   SET_E(); Wait_us(1);    //    E=1
   CLR_E(); Wait_us(40);   //  
   SET_E(); Wait_us(1);
   CLR_E(); Wait_us(40);   //  
   LCD_PORT &= 0xF0;          //  0x30
   LCD_PORT |= 0x02;          //   (4 ) (0x02 -    0x20) (   RW  A0   !)
   SET_E(); Wait_us(1);    //     (tDSW)
   CLR_E(); Wait_us(1);    //     E=1
   LCD_PORT &= 0xF0;
//               
   LCD_write_cmd(0x2A); Wait_us(40);  //   
   LCD_write_cmd(0x0C);	Wait_us(40);  // ,  
   cls();                             //        
   LCD_write_cmd(0x06); Wait_us(40);  //   :   
}

/****************************************
 *      *
 *    1,  -  0 *
 ****************************************/
void Locate(UINT8 line, UINT8 pos)
{
   UINT8 res;

   switch (line)
   {
      case 1:
         res = 0x80;
         break;
      case 2:
         res = 0xC0;
         break;
      case 3:
         res = 0x90;
         break;
      case 4:
         res = 0xD0;
         break;
      default:
         res = 0x80;
   }

   res |= pos;

   LCD_write_cmd(res);
}

/******************************************************************
 *                                           *
 ******************************************************************/
void LCDPortInit(void)
{
   LCD_PORT_DIR   |= _BV(LCD_PIN0)|_BV(LCD_PIN1)|_BV(LCD_PIN2)|_BV(LCD_PIN3)|_BV(LCD_PIN_STROBE)|_BV(LCD_PIN_RS);    //   LCD   !!!
   LCD_PORT       &= ~(_BV(LCD_PIN0)|_BV(LCD_PIN1)|_BV(LCD_PIN2)|_BV(LCD_PIN3)|_BV(LCD_PIN_STROBE)|_BV(LCD_PIN_RS)); //    LCD   0  !!!
}


/************************************************************
 *         *
 * pos -                          *
 ************************************************************/
void lcd_print_dec_float(char *text, UINT16 ciffer, UINT8 pos)
{
   UINT8         i;
   UINT8         counter;
   UINT8         print = 0;

   while (*text)
   {
      LCD_write_data((UINT8)*text++);
   }

   if (!ciffer)
   {
      lcd_print_str("0");
      return;
   }

   for (i = 0; i < 5; i++)
   {
      counter = 0;
      while (ciffer >= dec_mass_5[i])
      {
         ciffer -= dec_mass_5[i];
         counter++;
      }
      if (counter)
      {
         print = 1;
         LCD_write_data((UINT8)counter+0x30);
      }
      else if (print)
      {
         LCD_write_data((UINT8)counter+0x30);
      }
      if (4 == (i+pos))
      {
         if (1 != print) lcd_print_char('0');
         lcd_print_char('.');
      }
   }
}


/***********************************
 *          *
 ***********************************/
void buff_print_dec(char **buffer, UINT8 ciffer)
{
   UINT8         print = 0;
   UINT8         i;
   UINT8         counter;

   if (!ciffer)
   {
      *(*buffer)++ = '0'; //   0,     
      return;
   }

   for (i = 0; i < 2; i++)
   {
      counter = 0;
      while (ciffer >= dec_mass_2[i])
      {
         ciffer -= dec_mass_2[i];
         counter++;
      }
      if (counter)
      {
         print = 1;
         *(*buffer)++ = ((char)counter+0x30);
      }
      else if (print)
      {
         *(*buffer)++ = ((char)counter+0x30);
      }
   }
   **buffer = '\0';
}
